package com.example;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.impl.SLF4JLogFactory;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;

/**
 * @author Grigory Kalabin grigory.kalabin@gmail.com
 */
public class Listener implements ServletContextListener {
    private static final String JNDI_PREFIX = "java:jboss/";
    public static final String OBJECT_JNDI_NAME = JNDI_PREFIX + "secret";
    public static final String WORKER_JNDI_NAME = JNDI_PREFIX + "hardwork";
    private final Log logger = SLF4JLogFactory.getLog(getClass());

    @Override
    public void contextInitialized(ServletContextEvent sce) {
        try {
            InitialContext context = new InitialContext();
            context.bind(OBJECT_JNDI_NAME, "unicorn");
            WorkerThread target = new WorkerThread();
            context.bind(WORKER_JNDI_NAME, target);
            new Thread(target).start();
        } catch (NamingException e) {
            logger.error("unable to bind", e);
        }
    }

    @Override
    public void contextDestroyed(ServletContextEvent sce) {
        try {
            logger.info("found: " + InitialContext.doLookup(OBJECT_JNDI_NAME));
        } catch (NamingException e) {
            logger.error("unable to lookup", e);
        }

        try {
            ((WorkerThread) InitialContext.doLookup(WORKER_JNDI_NAME)).finish();
        } catch (NamingException e) {
            logger.error("unable to lookup", e);
        }
    }

    public final class WorkerThread implements Runnable {
        private boolean isRunning = true;

        @Override
        public void run() {
            while (isRunning) {
                logger.info("I'm still here");
                try {
                    Thread.sleep(1000);
                } catch (InterruptedException e) {
                    logger.info("ups", e);
                }
            }
        }

        public void finish() {
            logger.info("mkay");
            isRunning = false;
        }
    }
}
